###############################################################################
###############################################################################
##
##  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
##
##  This copyrighted material is made available to anyone wishing to use,
##  modify, copy, or redistribute it subject to the terms and conditions
##  of the GNU General Public License v.2.
##
###############################################################################
###############################################################################
%define _unpackaged_files_terminate_build 0

%define KERNEL_VERSION 2
%define KERNEL_PATCHLEVEL 4
%define KERNEL_SUBLEVEL 21
%define KERNEL_EXTRAVERSION 15.0.4.EL.XOS.1
%define KERNEL_EXTRAVERSION_RHEL 15.0.4.EL
%define kernel_version %{KERNEL_VERSION}.%{KERNEL_PATCHLEVEL}.%{KERNEL_SUBLEVEL}-%{KERNEL_EXTRAVERSION}
%define kernel_version_rhel %{KERNEL_VERSION}.%{KERNEL_PATCHLEVEL}.%{KERNEL_SUBLEVEL}-%{KERNEL_EXTRAVERSION_RHEL}


%define buildup 1
%define buildsmp 1
%define buildhugemem 1

%ifnarch i686
%define buildhugemem 0
%endif

%ifnarch i686 x86_64 athlon
%define buildsmp 0
%endif 


Name: GFS
Version: 6.0.0
Release: 7.1.XOS.1
License: GPL
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Obsoletes: GFS-gnbd GNBD
BuildRequires: rsync flex bison
BuildRequires: kernel-source = %{kernel_version}
ExcludeArch: i386 ppc ppc64 s390 s390x

Source0: gfs-build.tar.gz
Patch: GFS-6.0.0-chkconfig.patch

# Patches are applied later - after the source is extracted
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-buildroot

%description
This package contains the Global File System (GFS) and related packages.

%package devel
Summary: GFS Libraries and header files
GROUP: System Environment/Base

%description devel
Libraries and header files.

%if %{buildup}
%package modules
Requires: %{name} = %{version}-%{release}
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Requires: %{name} = %{version}-%{release}
Requires: kernel = %{kernel_version}

%description modules
This package contains the Global File System (GFS) kernel modules for a uniprocessor kernel.
%endif

%if %{buildsmp}
%package modules-smp
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Requires: %{name} = %{version}-%{release}
Requires: kernel-smp = %{kernel_version}

%description modules-smp
This package contains the Global File System (GFS) kernel modules for an SMP kernel.
%endif

%if %{buildhugemem}
%package modules-hugemem
GROUP: System Environment/Base
Summary: The GFS cluster file system and related products.
Requires: %{name} = %{version}-%{release}
Requires: kernel-hugemem = %{kernel_version}

%description modules-hugemem
This package contains the Global File System (GFS) kernel modules for a hugemem kernel.
%endif

####################################################################################
##  THE PREP SECTION
####################################################################################
%prep
echo ping
%setup -c -q
%patch -p0
for dir in {GFS,gnbd}/kernel_patches; do
	if [ ! -d "${dir}/%{kernel_version}" ]; then
		ln -s %{kernel_version_rhel} ${dir}/%{kernel_version}
	fi
done
echo pong
####################################################################################
##  THE BUILD SECTION
####################################################################################
%build
BUILD_TOPDIR=`pwd`

BuildSistina(){
	cpu_type=$1
	flavor=$2
	kernel_src=/usr/src/linux-%{kernel_version}
	# Kernel tree setup
	if [ -d $kernel_src/. ]; then
		echo "Kernel found."
	else
		echo "Kernel not found."
		ls /lib/modules/2.4.21-*EL
		exit 1
	fi

	rsync -aH $kernel_src/ linux$flavor
	make -C linux$flavor mrproper
	[ -d linux$flavor/configs ] || exit 1
	cat > linux$flavor/.config <<EOF
CONFIG_BLK_DEV_GNBD=m
CONFIG_BLK_DEV_POOL=m
CONFIG_LOCK_HARNESS=m
CONFIG_GFS_FS=m
CONFIG_LOCK_NOLOCK=m
CONFIG_LOCK_GULM=m
EOF
	cat linux$flavor/configs/kernel*${cpu_type}${flavor:+-}$flavor.config >> linux$flavor/.config
	for I in */kernel_patches/%{kernel_version}/*patch; do
		patch -d linux$flavor -p1 -f -i ../$I
	done
	make -C linux$flavor oldconfig \
		VERSION=%{KERNEL_VERSION} \
		PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
		SUBLEVEL=%{KERNEL_SUBLEVEL} \
		EXTRAVERSION=-%{KERNEL_EXTRAVERSION}$flavor

	make -C linux$flavor dep \
		VERSION=%{KERNEL_VERSION} \
		PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
		SUBLEVEL=%{KERNEL_SUBLEVEL} \
		EXTRAVERSION=-%{KERNEL_EXTRAVERSION}$flavor

	# Configure subdirs
	for I in GFS gnbd bedrock; do
		pushd $I
			./configure --mandir=%{_mandir} --kernel_src=$BUILD_TOPDIR/linux$flavor
		popd
	done
	pushd gulm
	./configure --mandir=%{_mandir} --kernel_src=$BUILD_TOPDIR/linux$flavor --libdir=%{_libdir}
	popd
	# Build it!
	make %{?_smp_mflags} ${flavor:-NO_FLAVOR}=y \
		VERSION=%{KERNEL_VERSION} \
		PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
		SUBLEVEL=%{KERNEL_SUBLEVEL} \
		EXTRAVERSION=-%{KERNEL_EXTRAVERSION}$flavor
}

%if %{buildhugemem}
BuildSistina %{_target_cpu} hugemem
%endif

%if %{buildsmp}
BuildSistina %{_target_cpu} smp
%endif

%if %{buildup}
BuildSistina %{_target_cpu}
%endif

####################################################################################
##  THE INSTALL SECTION
####################################################################################
%install
rm -rf $RPM_BUILD_ROOT
%if %{buildhugemem}
make install INSTALL_MOD_PATH=$RPM_BUILD_ROOT \
	DESTDIR=$RPM_BUILD_ROOT \
	VERSION=%{KERNEL_VERSION} \
	PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
	SUBLEVEL=%{KERNEL_SUBLEVEL} \
	EXTRAVERSION=-%{KERNEL_EXTRAVERSION}hugemem \
	hugemem=y
%endif

%if %{buildsmp}
make install INSTALL_MOD_PATH=$RPM_BUILD_ROOT \
	DESTDIR=$RPM_BUILD_ROOT \
	VERSION=%{KERNEL_VERSION} \
	PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
	SUBLEVEL=%{KERNEL_SUBLEVEL} \
	EXTRAVERSION=-%{KERNEL_EXTRAVERSION}smp \
	smp=y
%endif

%if %{buildup}
make install INSTALL_MOD_PATH=$RPM_BUILD_ROOT \
	DESTDIR=$RPM_BUILD_ROOT \
	VERSION=%{KERNEL_VERSION} \
	PATCHLEVEL=%{KERNEL_PATCHLEVEL} \
	SUBLEVEL=%{KERNEL_SUBLEVEL} \
	EXTRAVERSION=-%{KERNEL_EXTRAVERSION}
%endif

rm -rf $RPM_BUILD_ROOT/lib/modules/*/build
rm -rf $RPM_BUILD_ROOT/lib/modules/*/pcmcia



####################################################################################
##  THE CLEAN SECTION
####################################################################################
%clean
rm -rf $RPM_BUILD_ROOT


####################################################################################
##  THE POST SECTION
####################################################################################
%post
depmod -a
for i in ccsd pool gfs lock_gulmd; do
  /sbin/chkconfig --add $i
done

####################################################################################
##  THE PREUN SECTION
####################################################################################
%preun
if [ "$1" -eq 0 ]; then
for i in ccsd pool gfs lock_gulmd; do
  /sbin/chkconfig --del $i
done
fi

####################################################################################
##  THE POSTUN SECTION
####################################################################################
%postun
if [ -e /dev/poolcomm ] ; then
	rm -f /dev/poolcomm
fi

####################################################################################
##  THE FILES SECTION
####################################################################################
%files
%defattr(-,root,root,-)
# Binaries
/sbin/ccs_read
/sbin/ccs_servd
/sbin/ccs_tool
/sbin/ccsd
/sbin/fence_ack_manual
/sbin/fence_apc
/sbin/fence_brocade
/sbin/fence_egenera
/sbin/fence_gnbd
/sbin/fence_manual
/sbin/fence_mcdata
/sbin/fence_node
/sbin/fence_rib
/sbin/fence_vixel
/sbin/fence_wti
/sbin/fence_xcat
/sbin/fsck.gfs
/sbin/gfs_conf
/sbin/gfs_fsck
/sbin/gfs_grow
/sbin/gfs_jadd
/sbin/gfs_mkfs
/sbin/gfs_quota
/sbin/gfs_tool
/sbin/gnbd_export
/sbin/gnbd_import
/sbin/gulm_tool
/sbin/lock_gulmd
/sbin/mkfs.gfs
/sbin/pool_assemble
/sbin/pool_grow
/sbin/pool_info
/sbin/pool_mp
/sbin/pool_tool
/etc/init.d/ccsd
/etc/init.d/pool
/etc/init.d/gfs
/etc/init.d/gfs-functions
/etc/init.d/lock_gulmd

# Man pages
%{_mandir}/man4/pool.4.gz
%{_mandir}/man5/gfs_conf_examples.5.gz
%{_mandir}/man5/lock_gulmd.5.gz
%{_mandir}/man5/pool_conf.5.gz
%{_mandir}/man5/gfs_mount.5.gz
%{_mandir}/man8/ccs.8.gz
%{_mandir}/man8/ccs_read.8.gz
%{_mandir}/man8/ccs_servd.8.gz
%{_mandir}/man8/ccs_tool.8.gz
%{_mandir}/man8/ccsd.8.gz
%{_mandir}/man8/fence.8.gz
%{_mandir}/man8/fence_ack_manual.8.gz
%{_mandir}/man8/fence_apc.8.gz
%{_mandir}/man8/fence_brocade.8.gz
%{_mandir}/man8/fence_egenera.8.gz
%{_mandir}/man8/fence_gnbd.8.gz
%{_mandir}/man8/fence_manual.8.gz
%{_mandir}/man8/fence_mcdata.8.gz
%{_mandir}/man8/fence_node.8.gz
%{_mandir}/man8/fence_rib.8.gz
%{_mandir}/man8/fence_vixel.8.gz
%{_mandir}/man8/fence_wti.8.gz
%{_mandir}/man8/fence_xcat.8.gz
%{_mandir}/man8/gfs.8.gz
%{_mandir}/man8/gfs_conf.8.gz
%{_mandir}/man8/gfs_fsck.8.gz
%{_mandir}/man8/gfs_grow.8.gz
%{_mandir}/man8/gfs_jadd.8.gz
%{_mandir}/man8/gfs_mkfs.8.gz
%{_mandir}/man8/gfs_quota.8.gz
%{_mandir}/man8/gfs_tool.8.gz
%{_mandir}/man8/gnbd.8.gz
%{_mandir}/man8/gnbd_export.8.gz
%{_mandir}/man8/gnbd_import.8.gz
%{_mandir}/man8/gulm_tool.8.gz
%{_mandir}/man8/lock_gulmd.8.gz
%{_mandir}/man8/pool.8.gz
%{_mandir}/man8/pool_assemble.8.gz
%{_mandir}/man8/pool_grow.8.gz
%{_mandir}/man8/pool_info.8.gz
%{_mandir}/man8/pool_mp.8.gz
%{_mandir}/man8/pool_tool.8.gz

%if %{buildup}
%files modules
%defattr(-,root,root,-)
# Modules
/lib/modules/%{kernel_version}/kernel/drivers/block/gnbd/gnbd.o
/lib/modules/%{kernel_version}/kernel/drivers/block/gnbd/gnbd_serv.o
/lib/modules/%{kernel_version}/kernel/drivers/md/pool/pool.o
/lib/modules/%{kernel_version}/kernel/fs/gfs/gfs.o
/lib/modules/%{kernel_version}/kernel/fs/gfs_locking/lock_gulm/lock_gulm.o
/lib/modules/%{kernel_version}/kernel/fs/gfs_locking/lock_harness/lock_harness.o
/lib/modules/%{kernel_version}/kernel/fs/gfs_locking/lock_nolock/lock_nolock.o
%endif

%if %{buildsmp}
%files modules-smp
%defattr(-,root,root,-)
# Modules
/lib/modules/%{kernel_version}smp/kernel/drivers/block/gnbd/gnbd.o
/lib/modules/%{kernel_version}smp/kernel/drivers/block/gnbd/gnbd_serv.o
/lib/modules/%{kernel_version}smp/kernel/drivers/md/pool/pool.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs/gfs.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs_locking/lock_gulm/lock_gulm.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs_locking/lock_harness/lock_harness.o
/lib/modules/%{kernel_version}smp/kernel/fs/gfs_locking/lock_nolock/lock_nolock.o
%endif

%if %{buildhugemem}
%files modules-hugemem
%defattr(-,root,root,-)
# Modules
/lib/modules/%{kernel_version}hugemem/kernel/drivers/block/gnbd/gnbd.o
/lib/modules/%{kernel_version}hugemem/kernel/drivers/block/gnbd/gnbd_serv.o
/lib/modules/%{kernel_version}hugemem/kernel/drivers/md/pool/pool.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs/gfs.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs_locking/lock_gulm/lock_gulm.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs_locking/lock_harness/lock_harness.o
/lib/modules/%{kernel_version}hugemem/kernel/fs/gfs_locking/lock_nolock/lock_nolock.o
%endif

%files devel
%{_libdir}/libgulm.a
%{_includedir}/libgulm.h

%changelog
* Fri Aug 20 2004 X/OS Linux Team <xoslinux@xos.nl> 6.0.0-7.1.XOS.1
- Build for X/OS Linux 3.0.
- X/OS Linux source rpm based on GFS-6.0.0-7.1.src.rpm
  (md5sum of original source rpm: 92fc0140b5c03e94288ac944dfcf8258).
- Depend on X/OS Linux kernel release (macro KERNEL_EXTRAVERSION).
- Symlink directories with kernel patches if needed.

* Mon Aug 2 2004 Jonathon Brassow <jbrassow@redhat.com> 6.0.0-7.1
- Fix stack overflow issue in gulm on x86-64
- Rebuild for kernel 2.4.21-15.0.4.EL (a U2 errata kernel)
* Thu Jul 22 2004 Jonathon Brassow <jbrassow@redhat.com> 6.0.0-8
- Test build for U3
* Thu Jul 8 2004 AJ Lewis <alewis@redhat.com> 6.0.0-7
- Rebuild
* Thu Jul 8 2004 AJ Lewis <alewis@redhat.com> 6.0.0-6
- Actually include the fix that should have been in 6.0.0-5
* Thu Jul 8 2004 AJ Lewis <alewis@redhat.com> 6.0.0-5
- Additional fix for #127031 - Increase initial jid mapping pool size
* Fri Jul 2 2004 Jonathon Brassow <jbrassow@redhat.com> 6.0.0-4
- Fix for #127031 - Add unlock call to jid unlock callback in lockgulm
* Wed Jun 30 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-3
- Add gfs-functions file and update for kernel 2.4.21-15.0.3.EL
* Wed Jun 23 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-2
- Update for new kernel
* Wed Jun 2 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-1.2
- Fix problem with gnbd major number conflict causing kernel panic, final (sbz1333)
- Fix gulm issue that cause GFS to panic with 
  head.lh_flags & GFS_LOG_HEAD_UNMOUNT (sbz1298)
- Add ability for lock_gulmd to reconnect after it has been killed
  and restarted (sbz1265)
- Fix gnbd SMP issue (sbz1322)
- Fix gfs_mount typo (sbz1319)
- Fix gfs_fsck typo (sbz1327)
* Wed Jun 2 2004 Derek Anderson <danderso@redhat.com> 6.0.0-1.1
- Fix problem with gnbd major number conflict causing kernel panic (sbz1333)
* Fri May 28 2004 Derek Anderson <danderso@redhat.com> 6.0.0-1.0
- Fix problem with gulm generation id mismatch (sbz1300)
- Fix problem with gulm master fencing clients after loss of slave (sbz1297)
* Wed May 26 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.15
- Fix gfs_mount.8 typo (sbz1319)
- Fix lock_gulm issue that occurs if all servers die, again (sbz1300)
* Mon May 24 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.14
- Fix problem with preun script running chkconfig --del on upgrade (sbz1326)
- Fix gfs_fsck problem of counting EA blocks incorrectly (sbz1324)
- Fix gulm bug where it thinks a slave is logged in but isn't (sbz1294)
- Fix master gulm server fencing same node multiple times (sbz1313)
- Fix problem with lock_gulmd crashing when master server dies (sbz1288)
* Fri May 21 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.13
- Fix bug in GFS (rather make GFS work around kernel bug) that causes
  kernel panic with "list_empty(&inode->i_dirty_data_buffers)" sbz(1271)
- Fix problem with lock_gulmd hanging the kernel (sbz1316)
- Fix truncated init scripts definitions (sbz1282)
* Thu May 20 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.12
- Update fence agent man pages (sbz1318)
- CCS does not need to check for ip_interfaces in nodes.ccs (sbz1317)
- Fix GETBLKSIZE64 ioctl call in libiddev (sbz1301)
- Fix lock_gulm issue that occurs if all servers die (sbz1300)
- Fix ACL permissions problem (sbz1309)
- Add ACL mount option description to gfs_mount man page (sbz1308)
- Fix mount hang due to lock_gulm deadlock on recovery (sbz1258)
* Tue May 18 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.11
- Remove hardcoded patch directory to prevent out of sync builds
- gfs_fsck fix for extended attributes (sbz1273)
- Give pool the ability to do read-only devices (sbz1304)
* Mon May 10 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.10
- Rebuild for Pool grow bug fix (sbz1296)
- Increase the number of possible paths for Pool multipathing (sbz1302)
- Fix bug in fence were not all arguments were being passed to fence agent (sbz1299)
* Fri May 7 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.9
- build again, this time with i686 support
* Wed May 5 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.8
- Forgot to include the fence_mcdata man page
* Tue May 4 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.7
- New build for new collection instance
* Tue Apr 20 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.6
- Pull gfs_eattr and man page.  There are now native ways of doing this.
- Add init scripts for gfs, pool, ccsd, and lock_gulmd
* Mon Apr 19 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.5
- Add fence_xcat and its man page to the files list
- Turn on smp package build
* Fri Apr 16 2004 Jonathan Brassow <jbrassow@redhat.com> 6.0.0-0.5
- Move items around in the files section so everything is in the
  correct package
- Exclude certain archs which aren't supported
- Specify kernel version info to all make targets so that modules install
  to the correct spot
- Add --libdir arg to configure so gulm can put it's libs in the right spot
* Tue Apr 13 2004 Elliot Lee <sopwith@redhat.com> 6.0.0-0.1
- Avoid using /tmp/my_requires (security hole), and avoid filtering
  requires altogether, since the %pre scripts are trying to do the
  equivalent of dependencies anyways
- Remove Provides: kernel-modules (since this is probably not what is intended).
- On a wild guess, change Conflicts: to Obsoletes:. The theory here is
  that the functionality formerly in GFS-gnbd and GNBD is now
  contained in this package - for that situation, Obsoletes: is more
  appropriate.
- Rework the subpackages so that the userland utils are all in the main
  package, and the kernel modules are in subpackages (one per kernel
  variant).
- Use _smp_mflags to speed up the build on SMP machines.
- Eliminate perl as a build-time dependency by doing the equivalent
  of ./configure inside the .spec file in fewer lines.
- BuildRequires on kernel/kernel-smp/kernel-hugemem (must be unconditional)
